#!/bin/bash
# Nathan Oyama <nathan.oyama@ibm.com> - Wed Jan  9 07:42:30 STD 2019

TARGET_PATH="/opt/virgo/repository/ecx-usr" 
TEMP_PATH="/opt/virgo/repository/temp-properties"
SCRIPT_DIR=$(dirname "$(test -L "$0" && readlink "$0" || echo "$0")")



if [ "$1" == "-h" ] || [ "$1" == "--help" ]; then
  echo "Usage: sudo bash OLD_DAT NEW_DAT [DESTINATION]"
  echo "For IBM Spectrum Protect Plus, combine OLD_DAT and NEW_DAT, then "
  echo "produce it in [DESTINATION]."
  echo "Example 1. sudo bash ./isoCompMerger.sh \\"
  printf "  /opt/virgo/repository/temp-properties/"
  printf "properties.10.1.1-100.2019-01-01_0000utc.dat \\ \n"
  printf "  /opt/virgo/repository/temp-properties/"
  printf "properties.10.1.2-200.2019-01-02_0000utc.dat \\ \n"
  echo "Example 2. sudo bash ./isoCompMerger.sh \\"
  printf "  /opt/virgo/repository/temp-properties/"
  printf "properties.10.1.1-100.2019-01-01_0000utc.dat \\ \n"
  printf "  /opt/virgo/repository/temp-properties/"
  printf "properties.10.1.2-200.2019-01-02_0000utc.dat \\ \n"
  echo "  /tmp/"
  echo "where the third argument \"/tmp/\" for [DESTINATION] is optional."
  echo "The default [DESTINATION] is $TARGET_PATH ."
  echo "OLD_DAT and NEW_DAT can be generated by isoCompDuplicator.sh ."
  echo "Currently, you have the following DAT file(s) in $TARGET_PATH :"
  ls -1 $TEMP_PATH/properties*.dat
  exit $?
fi

echo "Checking your arguments..."
if [ "$EUID" -ne 0 ]; then
  echo "Permission denied. $(whoami), please use \"sudo\". Aborting."        >&2
  exit 1
fi

check_files () {
  LIST_FILE="$1"
  echo "Checking all the files listed in $(basename $LIST_FILE)."
  BAD_FILE=false
  cat "$LIST_FILE" | while read file; do
    if [ ! -f "$file" ]; then
      echo "Unable to find $file"                                            >&2
      BAD_FILE=true
    fi
    if $BAD_FILE; then
      echo "If you don't wish to use above files, manually delete the "        >&2
      echo "item(s) in $(basename $LIST_FILE) and retry. Also make sure this " >&2
      echo "is the correct file. Aborting."                                    >&2
      exit 1
    fi
  done
  echo "Confirmed all the files exist."
}

if [ -f "$1" ] && [[ "$(basename $1)" = properties*utc.dat ]]; then
    DAT_OLD="$(realpath $1)"
else
  echo "Invalid old file was given in the 1st argument."                     >&2
  echo "Try $ bash ./isoCompMerger.sh -h for more information. Aborting."    >&2
  exit 1
fi
check_files "$DAT_OLD"
echo "DAT_OLD: $DAT_OLD"

if [ -f "$2" ] && [[ "$(basename $2)" = properties*utc.dat ]]; then
    DAT_NEW="$(realpath $2)"
else
  echo "Invalid new file was given in the 2nd argument."                     >&2
  echo "Try $ bash ./isoCompMerger.sh -h for more information. Aborting."    >&2
  exit 1
fi
if $(cmp --silent $DAT_NEW $DAT_OLD); then
  echo "These two files are identical. There is nothing to do."
  exit 0
fi

if [[ $DAT_NEW < $DAT_OLD ]]; then
  echo "The file in the 1st argument looks newer. If it is actually older "  >&2
  echo "(mainly due to an incorrect system clock), please manually rename "  >&2
  echo "$(basename ${DAT_OLD}) . You don't have to update the duplicated "   >&2
  echo "files. Try $ bash ./isoCompMerger.sh -h for more information."       >&2
  echo "Aborting."                                                           >&2
  exit 1
fi
check_files "$DAT_NEW"
echo "DAT_NEW: $DAT_NEW"

if [ ! -z "$3" ]; then
  if [ ! -d "$3" ]; then
    echo "Invalid destination directory was given in the 3rd argument."      >&2
    echo "Try $ bash ./isoCompMerger.sh -h for more information. Aborting."  >&2
    exit 1
  fi
  TARGET_PATH=$(realpath $3)
fi
echo "TARGET_PATH: $TARGET_PATH"

echo "All the arguments look good."

EXCEPTION_CSV="${SCRIPT_DIR}/iso_exception_keys.csv"
if [ -e "$EXCEPTION_CSV" ]; then
  echo "Exception list: $(realpath $EXCEPTION_CSV)"
  cat "$EXCEPTION_CSV"
else
  echo "Unable to locate an exception file: $(realpath $EXCEPTION_CSV)"      >&2
  echo "Aborting."                                                           >&2
  exit 1
fi

echo -e "\nMerging files..."

FILE_LIST_OLD=$(rev $DAT_OLD | cut -d '.' -f 5- | rev)
FILE_LIST_NEW=$(rev $DAT_NEW | cut -d '.' -f 5- | rev)

echo -e "\nType 1: Files that only exist in DAT_OLD."
UNIQ_FILES_OLD="$(comm -23 <(echo "$FILE_LIST_OLD") <(echo "$FILE_LIST_NEW"))"
echo -e "\n$UNIQ_FILES_OLD\n"

if [ ! -z "$UNIQ_FILES_OLD" ]; then
  echo "Copy these file to $TARGET_PATH ..."
  echo "$UNIQ_FILES_OLD" | while read file
  do
    file_oldname=$(grep "$file" $DAT_OLD)
    cp -v "$file_oldname" "$TARGET_PATH/$(basename $file)"
  done
fi
echo "... done."

echo -e "\nType 2: Files that only exist in DAT_NEW."
echo "Do nothing and reuse those in TARGET_PATH."

echo -e "\nType 3: Files that exist in both DAT_OLD and DAT_NEW."
COMM_FILES=$(comm -12 <(echo "$FILE_LIST_OLD") <(echo "$FILE_LIST_NEW"))

MD5OLD=$(md5sum $(grep "$COMM_FILES" $DAT_OLD) | rev | cut -d '.' -f 5- | rev)
MD5NEW=$(md5sum $(grep "$COMM_FILES" $DAT_NEW) | rev | cut -d '.' -f 5- | rev)

echo -e "\nThe following file(s) in DAT_OLD and DAT_NEW remained the same."
comm -12 <(echo "$MD5OLD" | sort) <(echo "$MD5NEW" | sort) \
  | rev | cut -d ' ' -f 1 | rev | sort -u | sed "s|$TEMP_PATH|$TARGET_PATH|g"
echo "Do nothing and reuse those in TARGET_PATH."

echo -e "\nThe following file(s) in DAT_OLD and DAT_NEW are different."
COMM_FILES_DIFF="$(comm -3 <(echo "$MD5OLD" | sort) <(echo "$MD5NEW" | sort) \
    | rev | cut -d ' ' -f 1 | rev | sort -u)"
if [ -z "$COMM_FILES_DIFF" ]; then
  echo "(none)"
else
  echo "$COMM_FILES_DIFF" | while read file; do
    basename "$file"
  done
  
  echo ""
  echo "$COMM_FILES_DIFF" | while read file; do
    filename_target="$TARGET_PATH/$(basename $file)"
  
    filename_new="$(grep -m 1 "$file" $DAT_NEW)"
    filename_old="$(grep -m 1 "$file" $DAT_OLD)"
    file_mod=$(cat $filename_new)
    
    lines_old_uniq="$(comm -13 \
      <(sed "s/^[ ]\+//g" $filename_new | sort -u) \
      <(sed "s/^[ ]\+//g" $filename_old | sort -u))" 
    
    if [ -z "$lines_old_uniq" ]; then
      printf "All the lines in the old version present in the new one, so do "
      printf "nohing with $filename_target\n"
    else
      echo "Updating $filename_target"
      while read line; do
        key_old="$(awk -F '=' '/^[^ #=]+=[^ #=]+/ {print $1}' <<< "$line")"
        # Only pick the last matched record
        exception="$(tac $EXCEPTION_CSV \
          | grep -m 1 "^$(basename "$file")\,$key_old\,")"
        if [ ! -z "$exception" ]; then
          echo "  Exception. $exception"
          # The 3rd column may include comma
          exception_col3="$(cut -d ',' -f 3- <<< "$exception")"
          if [ "$exception_col3" = "KEEP" ]; then
            lines_old_uniq=$(sed "s|^[ ]*${key_old}.*||g" <<< "$lines_old_uniq")
          elif [ "$exception_col3" = "DELETE" ]; then
            file_mod=$(sed "s|^[ ]*${key_old}.*||g" <<< "$file_mod")
            lines_old_uniq=$(sed "s|^[ ]*${key_old}.*||g" <<< "$lines_old_uniq")
          else
            file_mod=$(sed "s|^[ ]*${key_old}.*||g" <<< "$file_mod")
            exception_col3=$(sed 's|\\|\\\\|g' <<< "$exception_col3") # for sed
            lines_old_uniq=$(\
              sed "s|^[ ]*${key_old}.*|${key_old}=$exception_col3|g" \
              <<< "$lines_old_uniq")
          fi
        else
          file_mod=$(sed "s|^[ ]*${key_old}.*||g" <<< "$file_mod")
        fi
      done <<< "$(grep "^[^ #=]\+=[^ #=]\+" <<< "$lines_old_uniq")"
  
#       file_mod=$(echo -E "$file_mod
# 
# # isoCompMerger DAT_OLD: $filename_old")
#       file_mod=$(echo -E "$file_mod
# # isoCompMerger DAT_NEW: $filename_new")

      echo -E "$file_mod" > $filename_target
      echo -E "$lines_old_uniq" >> $filename_target
    fi
  done
fi

echo -e "\nThe merging operation has been completed successfully."

exit $?

