#!/bin/bash

RUN_DATE=`date`
LOCAL_REPOFILE="/etc/yum.repos.d/local.repo"
IBM_REPOFILE="/etc/yum.repos.d/spp.repo"

MOUNT_POINT="/mnt/cslocalupdate"
YUM_REPO_DIRECTORY="/etc/yum.repos.d/"

if [ -d /data/log/adminconsole ]; then
    LOGFILE="/data/log/adminconsole/offline_update.log"
else
    LOGFILE="/tmp/offline_update.log"
fi

########################
# Failed Exit
########################
failed_exit() {
  inspect_rc 1
}

#########################################################
# inspect return code and return error if necessary
# ARG1 = Return code
# ARG2 = log file - File name path of the output for logs
#########################################################
inspect_rc() {
  RC=$1
  if [ $RC -eq 0 ]; then
    echo "" >> ${LOGFILE}
    echo "	done." >> ${LOGFILE}
  else
    echo "" >> ${LOGFILE}
    echo "	Error encountered.  RC=$RC" | tee -a ${LOGFILE}  >&2
    exit $RC
  fi
}

#####################################################
# Sets up yum repository environment
#####################################################
prepare_yum_repo() {
	echo "Preparing yum repository" >> ${LOGFILE}
	echo "Checking for mount point" >> ${LOGFILE}
	mount  | grep ${MOUNT_POINT} >/dev/null
	if [ $? -eq 0 ];then
		echo "Unmounting mount point ${MOUNT_POINT}" >> ${LOGFILE}
		umount ${MOUNT_POINT}
		inspect_rc $?
		echo "Cleaning up ${MOUNT_POINT}" >> ${LOGFILE}
		rm -rf ${MOUNT_POINT}
		inspect_rc $?
	fi
	echo "Creating mount directory" >> ${LOGFILE}
	mkdir -p ${MOUNT_POINT}
	inspect_rc $?
	echo "Setting up mount point" >> ${LOGFILE}	
	mount -o loop ${IMAGE_LOCATION}/${IMAGE_NAME} ${MOUNT_POINT}
	if [ $? -ne 0 ];then
		echo "Cant mount image ${IMAGE_LOCATION}/${IMAGE_NAME}" >> ${LOGFILE}
		failed_exit
	fi
	
	if [ -f ${LOCAL_REPOFILE} ]; then
		echo "Deleting ${LOCAL_REPOFILE} file" >> ${LOGFILE}
		rm -rf ${LOCAL_REPOFILE}
		inspect_rc $?
	fi

	#validate if ISO has expected structure
	if [ ! -d ${MOUNT_POINT}/SPP ] ; then
	    echo "Provided iso does not have expected file structure. Check for correct iso file. " | tee -a ${LOGFILE}  >&2
	    exit 1
	fi

	echo "Creating ${LOCAL_REPOFILE} file" >> ${LOGFILE}
	cat > $LOCAL_REPOFILE <<EOF
[localrepo]
name=Offline Local Repository
baseurl=file://${MOUNT_POINT}/SPP
enabled=1
gpgcheck=0
EOF
	if [ -d ${MOUNT_POINT}/OS/Packages/ ] ; then
	    cat >> $LOCAL_REPOFILE <<EOF


[osupdate]
name=Offline Local Repository for OS update
baseurl=file://${MOUNT_POINT}/OS
enabled=1
gpgcheck=0
EOF
	fi

	inspect_rc $?



}


#####################################################
# Cleanup after manual update process
#####################################################
cleanup_process() {
	echo "Cleanup process" >> ${LOGFILE}
	if [[ ${IMAGE_NAME} =~ ^spp-[ht]f.*\.rpm$ ]]; then
		echo "Cleanup for hotfix file" >> ${LOGFILE}
		rm -rf ${IMAGE_LOCATION}/${IMAGE_NAME}
		inspect_rc $?
		echo "Cleaning up hotfix directory" >> ${LOGFILE}
		rm -rf ${IMAGE_LOCATION}
		inspect_rc $?
	else
		echo "Cleanup for update image" >> ${LOGFILE}
		echo "Checking for mount point" >> ${LOGFILE}
		mount  | grep ${MOUNT_POINT} >/dev/null
		if [ $? -eq 0 ];then
			echo "Unmounting mount point ${MOUNT_POINT}" >> ${LOGFILE}
			umount ${MOUNT_POINT}
			inspect_rc $?
			echo "Cleaning up ${MOUNT_POINT}" >> ${LOGFILE}
			rm -rf ${MOUNT_POINT}
			inspect_rc $?
		fi

		if [ -f ${LOCAL_REPOFILE} ]; then
			echo "Deleting ${LOCAL_REPOFILE} file" >> ${LOGFILE}
			rm -rf ${LOCAL_REPOFILE}
			inspect_rc $?
		fi

		echo "Cleaning up image file" >> ${LOGFILE}
		rm -rf ${IMAGE_LOCATION}/${IMAGE_NAME}
		inspect_rc $?
		echo "Cleaning up image directory" >> ${LOGFILE}
		rm -rf ${IMAGE_LOCATION}
		inspect_rc $?
	fi
}

delete_old_image_if_any(){
    find  /tmp/ecx* -name spp-update.iso -mmin +30  -exec rm {} \; -print 2>/dev/null 
}


######################
# Process Upload file
######################
process_upload(){
	if [ -f ${IMAGE_LOCATION}/${IMAGE_NAME} ]; then
		if [[ ${IMAGE_NAME} =~ .*\.iso$ ]]; then
			echo "ISO file detected" >> ${LOGFILE}
			prepare_yum_repo
			echo "Clean yum cache" >> ${LOGFILE}
			yum clean all

			# todo
			# add validation

		elif [[ ${IMAGE_NAME} =~ ^spp-[hh]f.*\.rpm$ ]]; then
			echo "Hotfix RPM ${IMAGE_NAME} detected under ${IMAGE_LOCATION}" >> ${LOGFILE}
			echo "Validating ${IMAGE_NAME}"

			RPM_NAME=$(rpm -qp --queryformat "%{NAME}" ${IMAGE_NAME} )
			if [[ $a  =~ spp-[ht]f ]]; then
				#returns 200 to indicate hotfix upload
				echo "Valid hotfix/testfix RPM file detected" >> ${LOGFILE}
				exit 200
			else
				echo "Validation for hotfix/testfix rpm failed" | tee -a ${LOGFILE}  >&2
				exit 2
			fi
		else
			echo "Unrecognized file was uploaded" | tee -a ${LOGFILE}  >&2
			exit 2
		fi
	else
		echo "${IMAGE_LOCATION}/${IMAGE_NAME} does not exist" | tee -a ${LOGFILE}  >&2
		exit 1
	fi
}

###########################################
# This is where the script starts
###########################################
if [ $# -eq 0 ]; then
	exit 1
else
	if [ $# -lt 6 ]; then
		echo "$0: not enough arguments" | tee -a ${LOGFILE}  >&2
		exit 1
	elif [ $# -gt 6 ]; then
		echo "$0: too many arguments" | tee -a ${LOGFILE}  >&2
		exit 1
	else
		echo "" >> ${LOGFILE}
		echo "" >> ${LOGFILE}
		echo "##########################################" >> ${LOGFILE}
		echo "Manual Update Process Setup" >> ${LOGFILE}
		echo "" >> ${LOGFILE}
		echo "RUN DATE/TIME: ${RUN_DATE}" >> ${LOGFILE}
		echo "" >> ${LOGFILE}

		while getopts ":m:n:l:" opt; do
		  case $opt in
		    m)
		      echo "Setting mode" >> ${LOGFILE}
		      MODE=$OPTARG
		      inspect_rc $?      
		      ;;					
		    n)
		      echo "Setting image name" >> ${LOGFILE}
		      IMAGE_NAME=$OPTARG
		      inspect_rc $?      
		      ;;
		    l)
		      echo "Setting image location" >> ${LOGFILE}
		      IMAGE_LOCATION=$OPTARG
		      inspect_rc $?
		      ;;
		    \?)
		      echo "Invalid option: -$OPTARG" | tee -a ${LOGFILE}  >&2
		      failed_exit
		      ;;
		    :)
		      echo "Option -$OPTARG requires an argument." | tee -a ${LOGFILE}  >&2
		      failed_exit
		      ;;
		  esac
		done
		
		echo "MODE: ${MODE}" >> ${LOGFILE}
		echo "IMAGE NAME: ${IMAGE_NAME}" >> ${LOGFILE}
		echo "IMAGE LOCATION: ${IMAGE_LOCATION}" >> ${LOGFILE}
		
		if [ ${MODE} == "setup" ]; then
		    delete_old_image_if_any
			process_upload
		elif [ ${MODE} == "cleanup" ]; then
			cleanup_process
			echo "Exiting script." >> ${LOGFILE}
			exit 0
		fi
	fi
fi
