#!/bin/bash

# This script considers the following scenarios:
# 1. 10.1.7 and 10.1.8 freshly deployed appliances, which have the data directory /data/postgres.
# 2. All appliances which have PostgreSQL version 9.6.18 or above.
# In case of MAJOR UPGRADE from 9.6, we would need a new script to accomodate this. For example: update/update_postgres92.sh file
# was used to upgrade appliances from 9.2.24 to 9.6.18. 
# For future reference: Documentaion for upgrading PGSQL to v10 https://www.postgresql.org/docs/10/upgrading.html

PGDATA_OLD=/data/postgres
PGDATA_NEW=/data/postgres_96
PG_VERSION=9.6.
PG_VERSION_INSTALLED=$(rpm -qa --queryformat "%{VERSION}\n" postgresql96)

# Check if PostgreSQL version is 9.6.x, else exit
if ! [[ $PG_VERSION_INSTALLED =~ $PG_VERSION ]]; then 
    echo "The postgres version is $VERSION, skip upgrade";
    exit 0
fi

# If /data/postgres_96 directory is not present, then create the directory.
if [ ! -d $PGDATA_NEW ]; then
  echo "==> Changing the data directory from /data/postgres to /data/postgres_96"
  cp -a $PGDATA_OLD $PGDATA_NEW
  echo "==> Renaming the previous Data directory to /data/postgres.bak"
  mv $PGDATA_OLD "${PGDATA_OLD}.bak"
fi


# Check the main PostgreSQL-9.6 service file and make sure PGDATA is pointing to /data/postgres_96
if grep -Fxq "Environment=PGDATA=/data/postgres_96" /usr/lib/systemd/system/postgresql-9.6.service
then
    echo "==> PGDATA is set to /data/postgres_96. No changes are required."
else
    echo "==> PGDATA is incorrect. Updating PGDATA path..."
    sudo sed -i 's#Environment=PGDATA=.*#Environment=PGDATA=/data/postgres_96#g' /usr/lib/systemd/system/postgresql-9.6.service
    echo "==> Reloading Daemon..."
    sudo systemctl daemon-reload
fi

# Check .bash_profile PGDATA specifications.
if grep -Fxq "PGDATA=/data/postgres_96" /var/lib/pgsql/.bash_profile
then
    echo "==> No changes required in the /var/lib/pgsql/.bash_profile"
else
    echo "==> Updating /var/lib/pgsql/.bash_profile"
    sudo sed -i 's#PGDATA=.*#PGDATA=/data/postgres_96#g' /var/lib/pgsql/.bash_profile
fi

# Update max_connection to 500. The default setting is max_connections=100
if grep -Fxq "max_connections = 500" /data/postgres_96/postgresql.conf
then
    echo "==> No changes required in the /data/postgres_96/postgresql.conf"
else
    echo "==> Updating /data/postgres_96/postgresql.conf..."
    sudo  sed  -i.bak -e   's/^max_connections = .\+/max_connections = 500/g' /data/postgres_96/postgresql.conf
fi

echo "==> Change authentication method to trust based."
sed -i.bak -e 's#peer$#trust#g' -e 's#ident$#trust#g'  /data/postgres_96/pg_hba.conf

echo "==> Restart and enable PostgreSQL-9.6"
sudo systemctl  restart  postgresql-9.6
sudo systemctl  enable postgresql-9.6
